
""" PyJSG -- Python JSON Schema Grammar Bindings

This package translates between JSON and Python objects, following the rules
specified by {JSON Schema Grammar<http://github.com/ericprud/jsglib>}

"""

__version__ = '0.10.0'
__url__ = 'http://github.com/hsolbrig/pyjsg'
__license__ = 'CC0 1.0 Universal'

